<?php
namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\HTTP\Kernel;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Geral\Repository\AquisicaoProdutoRuralRepository;
use HCM\Util\Repositories;

use HCM\FPA\Services\ExportacaoExcelService;

class AquisicaoProdutoRuralService extends Crud {
    
    protected $dataSourceName = 'eso_aquisicaoprodutorural_get';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');
    
    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, AquisicaoProdutoRuralRepository $aquisicaoProdutoRuralRepository, ExportacaoExcelService $exportacaoExcelService, Kernel $httpKernel, EntityManager $entityManager) {
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->aquisicaoProdutoRuralRepository  = $aquisicaoProdutoRuralRepository;
        $this->exportacaoExcelService = $exportacaoExcelService;
        $this->httpKernel = $httpKernel;
        $this->entityManager = $entityManager;
        parent::__construct($dataSourceManager);
    }
    
    public function importarPlanilhaAquisicaoProdutoRural($row) {
        //Dados básicos
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
        $messages = array(); //Mensages de erro, se estiver vazio não houve erros
        
        //Aquisição dos dados de exportação
        $dados = $this->getDadosPlanilha($row['DSPATH'][0]);

        foreach($dados as $lin => $dado) {
            
            $estrutempreg = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura(
                $nrorg, $dado['nrInsc'], $this->hcmParameters->getTipoEstruturaLegalOrg(), $dado['Competência']
            );
            if(empty($estrutempreg)){
                $messages[] = "Estrutura Legal com o cnpj ".$dado['nrInsc']." não encontrada.";
            }
            
            $estrutestab = $this->entityManager->getRepository(Repositories::ESTRUTURAH)->retornaEstruturaPorInscricaoTipoEstrutura(
                $nrorg, $dado['Nro de inscrição Adquirente'], $this->hcmParameters->getTipoEstruturaLegalOrg(), $dado['Competência']
            );
            if(empty($estrutempreg)){
                $messages[] = "Estrutura Legal com o cnpj ".$dado['Nro de inscrição Adquirente']." não encontrada.";
            }
            
            $nrestrutempreg = $estrutempreg[0]->getNrestruturam();
            $nrestrutestab = $estrutestab[0]->getNrestruturam();
            
            $nraquiprodrural = \HCM\Util\NovoCodigo::geraCodigo($nrorg, 'ESO_AQUIPRODRURAL', 1, 12, true);
            
            $res = $this->aquisicaoProdutoRuralRepository->insereAquisicaoProdutoRural(
                $nrorg,
                $nraquiprodrural,
                $dado['Competência'],
                $nrestrutempreg,
                $nrestrutestab,
                $dado['indAquis'],
                $dado['Tipo de Inscrição do Produtor'],
                $dado['Nro Inscrição do Produtor'],
                $dado['Valor Bruto da NF'],
                $dado['Valor da Contrib Previdenciaria'],
                $dado['Forma da tributação']
            );
        }
        
        return $messages;
    }
    
    public function getDadosPlanilha($path) {
        //Os ID's (campos) atribuídos aos dados da planilha
        $head = array(
            'tplnsc',
            'nrInsc',
            'Tipo de Inscrição Adquirente',
            'Nro de inscrição Adquirente',
            'indAquis',
            'Valor Bruto da NF',
            'Valor da Contrib Previdenciaria',
            'Forma da tributação',
            'Tipo de Inscrição do Produtor',
            'Nro Inscrição do Produtor',
            'Competência'
            );
        //Colunas de data
        $dateColumns = array('Competência');
        
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        
        $file->setActiveSheetIndex(0);

        return $this->exportacaoExcelService->preparaInfo_generic($file, $head, 'tplnsc', $dateColumns);
    }
    
}