<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 28/03/2016
 * Time: 12:40
 */

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment; 
use Geral\Entities\Operador;
use HCM\Util\DateUtil;
use HCM\Geral\Repository\OperadorRepository;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;

class OperadorService {

    const SITUACAO_ATIVO = 'A';

    protected $entityManager;
    protected $operador;
    /** @var Environment */
    protected $environment;

    public function __construct(EntityManager $entityManager, Environment $environment, OperadorRepository $operadorRepository, HCMParameters $hcmParameters) {
        $this->operadorRepository = $operadorRepository;
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->hcmParameters = $hcmParameters;

        
        $this->operador = REPOSITORIES::OPERADOR;
    }

    public function getNextId() {
        $idRow = $this->operadorRepository->getMaxId();
        $id = $idRow[0]['MAX_ID'];
        return $id;
    }

    private function retornaOperador($cdoperador) {
        if (is_numeric($cdoperador)) {
            $cdoperador = str_pad($cdoperador, 12, "0", STR_PAD_LEFT);
            $operador = $this->entityManager->getRepository('\Geral\Entities\Operador')
                ->findOneBy(array('cdoperador' => $cdoperador));
        }

        if (!isset($operador) || !$operador) {
            throw new \Exception("Operador não está cadastrado.");
        }

        return $this->operador = $operador;
    }

    private function retornaOperadoresOrg($nrorg) {
        $operadores = $this->entityManager->getRepository('\Geral\Entities\Operador')
                ->findBy(array('nrorg' => $nrorg));

        return $operadores;
    }

    private function validaParametros(Operador $operador) {
        // verifica situação do operador
        if($operador->getIdoperativo() !== self::SITUACAO_ATIVO) {
            throw new \Exception("Operador não está ativo.");
        }
        
        $nrorgTrab = $operador->getNrorgtrab();
        
        // verifica a organização do operador
        if(!isset($nrorgTrab)) {
            throw new \Exception("Operador não possui organização de trabalho.");
        }
    }

    private function retornaParametrosOperador(Operador $operador) {
        return $this->entityManager->getRepository('\Geral\Entities\GpeParamoper')
            ->findOneBy(array("cdoperador" => $operador->getCdoperador(), "nrorg" => $operador->getNrorg()));
    }

    private function retornaParametrosGerais(Operador $operador) {
        return $this->entityManager->getRepository('\Geral\Entities\Paramgeral')
            ->findOneBy(array("nrorg" => $operador->getNrorgtrab()));
    }

    private function retornaOrganizacao(Operador $operador){
        return $this->entityManager->getRepository('\Geral\Entities\Organizacao')
            ->findOneBy(array("nrorg" => $operador->getNrorgtrab()));
    }

    private function retornaParametrosOrganizacao(Operador $operador){
        return $this->entityManager->getRepository('\Teknisa\Model\Gestaopessoas\Entities\GpeParamorg')
            ->findOneBy(array("nrorg" => $operador->getNrorgtrab()));
    }
    
    private function retornaValorSalarioMinimo(Operador $operador, $dtmescompetenc, $paramorg){
        return $this->entityManager->getRepository('\Teknisa\Model\Gestaopessoas\Entities\FpaValorreferenciam')
            ->buscavalor($operador->getNrorgtrab(), $paramorg->getNrorgpadrao(), $dtmescompetenc, 'SALARIO_MINIMO');
    }

    public function factoryOperador($cdoperador) {
        $operador = $this->retornaOperador($cdoperador);

        $this->validaParametros($operador);

        /** @var $paramgeral \Geral\Entities\Paramgeral */
        $paramgeral  = $this->retornaParametrosGerais($operador);
        /** @var $paramoper \Geral\Entities\GpeParamoper */
        $paramoper   = $this->retornaParametrosOperador($operador);
        /** @var $organizacao \Geral\Entities\Organizacao */
        $organizacao = $this->retornaOrganizacao($operador);
        /** @var $paramorg \Folha\Entities\GpeParamorg */
        $paramorg = $this->retornaParametrosOrganizacao($operador);
        /** @var $valorReferencia  \Gestaopessoas\Entities\FpaValorreferenciam */
        $valorSalarioMin = $this->retornaValorSalarioMinimo($operador, $paramoper->getDtmescompetenc()->format('d/m/Y'), $paramorg);

        return array(
            "cdoperador"        => $operador->getCdoperador(),
            "nmoperador"        => $operador->getNmoperador(),
            "nrestruttrab"      => $operador->getNrestruttrab(),
            "nrorgpadrao"       => strval($paramorg->getNrorgpadrao()),
            "nrorg"             => strval($operador->getNrorg()),
            "nrorgtrab"         => strval($operador->getNrorgtrab()),
            "nmorganizacaotrab" => $organizacao->getNmorganizacao(),
            "dtmescompetenc"    => $paramoper->getDtmescompetenc()->format('d/m/Y'),
            "dtmescompetencdiacorrente" => $this->getDtmescomptencDiaCorrente(),
            "nrtipoestrutlegal" => $paramorg->getNrtpestrutlegal(),
            "idrealizacontab" => $paramorg->getIdrealizacontab(),
            "idrealizadespesa" => $paramorg->getIdrealizadespesa(),
            "nrtipoestrutgeren" => $paramorg->getNrtpestrutgeren(),
            "idtpcompetencia"   => $paramoper->getIdtpdtcompete(),
            "idambientesal"     => $paramoper->getIdambientesal(),
            "nrocorrecal"       => $paramoper->getNrocorrecal(),
            "nrtpmodalidcal"    => $paramoper->getNrtpmodalidcal(),
            "nrtpmodalidsal"    => $paramoper->getNrtpmodalidsal(),
            "dataatual"         => $this->getDataAtual(),
            "dtultimodiames"    => $this->getUltimoDiaMes(),
            "dtultimodiacompetencia" => $this->getUltimoDiaCompetencia($paramoper->getDtmescompetenc()),
            "cdpictcctb" => isset($paramgeral) ? $paramgeral->getCdpictcctb() : null,
            "idademincpfdep"    => $this->getIdadeMinCpf($paramoper, $paramorg),
            "nrtipoestrutomador" => $paramorg->getNrtpestrutomador(),
            "salariominimo"     => $valorSalarioMin 
        );
    }

    public function carregaOperadoresOrg($nrorg) {

        $operadores = $this->retornaOperadoresOrg($nrorg);

        $objOperadores = array();
        foreach ($operadores as $operador) {
            $objOperadores[] = array(
                "cdoperador" => $operador->getCdoperador(),
                "nmoperador" => $operador->getNmoperador()
            );
        }

        return $objOperadores;
    }

    public function updateNrorgtrabalhoOper($dataset) {
        $this->entityManager->beginTransaction();

        $operador = $this->retornaOperador($dataset["CDOPERADOR"]);
        $operador->setNrorgtrab($dataset["NRORGTRAB"]);
        $this->entityManager->persist($operador);
        $this->entityManager->flush();
        $this->entityManager->commit();
    }

    public function getDtmescomptencDiaCorrente() {
        return DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO);
    }

    public function getDataAtual() {
        return DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO);
    }

    public function getUltimoDiaMes() {
        return DateUtil::getUltimoDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO);
    }

    public function getUltimoDiaCompetencia($date) {
        return DateUtil::getUltimoDiaDoMes($date)->format(DateUtil::FORMATO_BRASILEIRO);
    }
    
    public function getIdadeMinCpf($paramoper, $paramorg) {
        return $this->entityManager->getRepository(Repositories::FPA_VALORREFERENCIAM)
            ->buscavalor($paramorg->getNrorg(), $paramorg->getNrorgpadrao(), $paramoper->getDtmescompetenc()->format('d/m/Y'), "IDADE_MIN_CPF_DEP");
    }
    
    public function novo($nrorgtrab,$nmoperador,$dtnascimento,$nmacesso,$nrparcnegocio,$senha,$idoperativo = null,$idstatusenha = null,
                        $idatuoperador = null,$idcadpraprovore = null,$cdfunc = null,
                        $idcadprproceore = null,$idligextoper = null,
                        $idsenhvardia = null,$idopermaster = null,$idoperacesso = null)
    {
        
        $operador = new $this->operador;
        $operador->setNrorg($nrorgtrab);
        $operador->setIdoperativo($idoperativo);
        $operador->setIdstatusenha($idstatusenha);
        $operador->setIdatuoperador($idatuoperador);
        $operador->setIdcadpraprovore($idcadpraprovore);
        $operador->setDtultatuoper(DateUtil::getDataAtual());
        $operador->setNrorgtrab($nrorgtrab);
        $operador->setCdfunc($cdfunc);
        $operador->setDtnascimento($dtnascimento);
        $operador->setNmacesso($nmacesso);
        $operador->setDsemailoper($nmacesso);
        $operador->setIdcadprproceore($idcadprproceore);
        $operador->setIdligextoper($idligextoper);
        $operador->setIdsenhvardia($idsenhvardia);
        $operador->setIdopermaster($idopermaster);
        $operador->setIdoperacesso($idoperacesso);
        $operador->setNrparcnegocio($nrparcnegocio);
        $operador->setNmoperador($nmoperador);
        $operador->setCdsenhaoperweb($senha);
        $operador->setDtinclusao(DateUtil::getDataAtual());
        $operador->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $operador->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $operador->setDtultatu(DateUtil::getDataAtual());
        $operador->setCdoperultatu($this->hcmParameters->getCdoperador());
        $operador->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $operador->setCdoperador(str_pad((string)$this->getNextId(), 12, "0", STR_PAD_LEFT));
        $this->entityManager->persist($operador);
        return $operador;
    }
    
    public function retornaOperadorOrgParcnegocio($nrorg, $nrparcnegocio) {
        $params = array('P_NRORG' => $nrorg, 'P_NRPARCNEGOCIO' => $nrparcnegocio);
        $operador = $this->operadorRepository->retornaOperadorOrgParcnegocio($params);

        return $operador;
    }

}