/*******************************************************************************************/
/* Database name: TEKNISA_SAAS                                                             */
/* Trigger: PTO_OCORRENCIA_ATESTADO                                                        */
/* Tabelas: GPE_ALTESITUFUNC e PTO_PROGOCORRENCIA                                          */
/* Tarefa: 305720	 - Atestado de Frequência                                              */
/*                                                                                         */
/* Criação:                                                                                */
/* 09/12/2020 - Marcelo Silva                                                              */
/* Alterado em 03/03/2021 para inclusão de nova situação funcional (5) na trigger		   */
/*                                                                                         */
/*******************************************************************************************/

CREATE OR REPLACE 
TRIGGER PTO_OCORRENCIA_ATESTADO
AFTER INSERT OR UPDATE OR DELETE
ON GPE_ALTESITUFUNC FOR EACH ROW
DECLARE  
  V_NRMOTIVOOCORFREQ_FERIAS NUMBER := 5;
  V_NRSITUFREQUENCIA_FERIAS NUMBER := 21000;
  V_NRTIPOOCOR_ABONOFALTA NUMBER := 5;
  V_NRVINCULOM NUMBER;
  V_NRORG NUMBER;
  V_PRXCODOCOR VARCHAR2(20);
  V_NRSITUFUNCM NUMBER;
BEGIN
	/* Insere as programação de ocorrência para abonar falta no período de atestado médico */
  
	IF (INSERTING OR UPDATING) THEN
	   V_NRORG    := :NEW.NRORG;
	   V_NRVINCULOM := :NEW.NRVINCULOM;
	   V_NRSITUFUNCM := :NEW.NRSITUFUNCM;
	ELSIF DELETING THEN
	   V_NRORG    := :OLD.NRORG;
	   V_NRVINCULOM := :OLD.NRVINCULOM;		 
	   V_NRSITUFUNCM := :OLD.NRSITUFUNCM;	   
	END IF;
        
	IF (V_NRORG = 1664) AND V_NRSITUFUNCM IN (5,12,15,16, 100042) THEN 	
				
		IF INSERTING THEN
		
			/* Insere programação de ocorrência no período do atestado */   
			NOVO_CODIGO_SC('PTO_PROGOCORRENCIA', V_PRXCODOCOR, 1, :NEW.NRORG);
			INSERT INTO PTO_PROGOCORRENCIA (NRORG,NRPROGOCORRENCIA,NRVINCULOM,DTINICIOPROGOCOR,DTFIMPROGOCOR,IDDIASSEMANA,IDTIPODIACALENDARIO,IDTIPOHORARIO,NRMOTIVOOCORFREQ,NRHORDIAM,NRSITUFREQUENCIA,NRSITUFREQGATILHO,QTMINGATILHOPROGOCOR,QTMAXGATILHOPROGOCOR,VRCOEFICIENTEACRESCIMO,IDPROGOCORRENCIA,QTMINIMAPROGOCOR,QTMAXIMAPROGOCOR,DSOBSERVACAO,IDATIVO,NRORGINCLUSAO,DTINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,DTULTATU,CDOPERULTATU,NRTIPOPROGOCORRENCIA,NRMARCACAOPONTO,NRINTERVALOHORARIO,IDPROGOCORCONFIRMADA,CDOPERVERIFICADO,DSOBSERVACAORECUSA) 
				VALUES (:NEW.NRORG, TO_NUMBER(V_PRXCODOCOR), V_NRVINCULOM, :NEW.DTINISITUFUNC, :NEW.DTFIMSITUFUNC, '1111111', NULL, 'N', V_NRMOTIVOOCORFREQ_FERIAS, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 'S', :NEW.NRORG, SYSDATE, :NEW.CDOPERINCLUSAO, :NEW.NRORG, SYSDATE, :NEW.CDOPERINCLUSAO, V_NRTIPOOCOR_ABONOFALTA, NULL, NULL, 'S', NULL, NULL);
		
		ELSIF UPDATING THEN
		
			/* Atualiza dados da programação de ocorrência no período do atestado */
			UPDATE PTO_PROGOCORRENCIA SET DTINICIOPROGOCOR = :NEW.DTINISITUFUNC, DTFIMPROGOCOR = :NEW.DTFIMSITUFUNC
			WHERE NRORG                = :OLD.NRORG
			AND NRVINCULOM           = V_NRVINCULOM
			AND DTINICIOPROGOCOR     = :OLD.DTINISITUFUNC
			AND DTFIMPROGOCOR        = :OLD.DTFIMSITUFUNC
			AND NRMOTIVOOCORFREQ     = V_NRMOTIVOOCORFREQ_FERIAS
			AND NRTIPOPROGOCORRENCIA = V_NRTIPOOCOR_ABONOFALTA;
		
		ELSIF DELETING THEN
		
			/* Exclui programação de ocorrência no período do atestado */
			DELETE FROM PTO_PROGOCORRENCIA
			WHERE NRORG              = V_NRORG
			AND NRVINCULOM           = V_NRVINCULOM
			AND DTINICIOPROGOCOR     = :OLD.DTINISITUFUNC
			AND DTFIMPROGOCOR        = :OLD.DTFIMSITUFUNC
			AND NRMOTIVOOCORFREQ     = V_NRMOTIVOOCORFREQ_FERIAS
			AND NRTIPOPROGOCORRENCIA = V_NRTIPOOCOR_ABONOFALTA;
		
		END IF;
	END IF;		
END;
/

CREATE OR REPLACE TRIGGER PTO_GERA_SITUFUNC_OCORRENCIA
AFTER INSERT OR UPDATE OR DELETE
ON PTO_PROGOCORRENCIA FOR EACH ROW
DECLARE  
  V_NRMOTIVOOCORFREQ NUMBER := NULL;
  V_NRMOTIVOAFAST    NUMBER := NULL;
  V_NRSITUFUNCM      NUMBER := NULL;
  V_NRPROXSITUFUNC   NUMBER := 1;
  V_NRQTDIASITUFUNC  NUMBER := NULL;
  V_PRXCODOCOR       VARCHAR2(20);
BEGIN           
  IF (NVL(:NEW.NRORG, 0) <> 1664 AND NVL(:OLD.NRORG, 0) <> 1664) THEN 	
	/* insere a situação funcional apenas se a ocorrência for confirmada/aprovada */
	IF :NEW.IDPROGOCORCONFIRMADA = 'S' THEN
		/* define depara dos motivos de ocorrência com os motivos de afastamento */
		CASE :NEW.NRMOTIVOOCORFREQ
		WHEN 5  THEN V_NRMOTIVOAFAST := 22;
		ELSE V_NRMOTIVOAFAST := NULL;
		END CASE;
	
		/* define depara dos motivos de ocorrência com as situações funcionais */
	
		CASE :NEW.NRMOTIVOOCORFREQ
		WHEN 5  THEN V_NRSITUFUNCM := 12;
		ELSE V_NRSITUFUNCM := NULL;
		END CASE;
	
		/* insere a situação funcional apenas se for encontrado o DePara por ser campo obrigatório */
		IF V_NRSITUFUNCM IS NOT NULL THEN
		IF INSERTING THEN
			/* Insere situação funcional referente a ocorrência */   
			NOVO_CODIGO_SC('GPE_ALTESITUFUNC', V_PRXCODOCOR, 1, :NEW.NRORG);
			INSERT INTO GPE_ALTESITUFUNC (NRALTESITUFUNC,NRORG,NRMOTIVOAFASTA,NRPROXSITUFUNC,CDTABECDI,CDDIAGNOST,NRSITUFUNCM,NRVINCULOM,DTINISITUFUNC,DTFIMSITUFUNC,DTVERIFSITUFUNC,DTPREVIRETOSITU,DTPRORRSITUFUNC,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,DSOBSALTSITFUNC,NRQTDIASITUFUNC,DTESTABSITUFUNC) 
				VALUES ((SELECT MAX(NRALTESITUFUNC) + 1 FROM GPE_ALTESITUFUNC WHERE NRORG = :NEW.NRORG)/*TO_NUMBER(V_PRXCODOCOR)*/, :NEW.NRORG, V_NRMOTIVOAFAST, V_NRPROXSITUFUNC, NULL, NULL, V_NRSITUFUNCM, :NEW.NRVINCULOM, :NEW.DTINICIOPROGOCOR, :NEW.DTFIMPROGOCOR, NULL, NULL, NULL, SYSDATE, SYSDATE, :NEW.NRORG, :NEW.CDOPERINCLUSAO, NULL, NULL, :NEW.DSOBSERVACAO, NULL, NULL);
	
			/* Insere situação funcional referente volta em atividade normal */   
			NOVO_CODIGO_SC('GPE_ALTESITUFUNC', V_PRXCODOCOR, 1, :NEW.NRORG);
			INSERT INTO GPE_ALTESITUFUNC (NRALTESITUFUNC,NRORG,NRMOTIVOAFASTA,NRPROXSITUFUNC,CDTABECDI,CDDIAGNOST,NRSITUFUNCM,NRVINCULOM,DTINISITUFUNC,DTFIMSITUFUNC,DTVERIFSITUFUNC,DTPREVIRETOSITU,DTPRORRSITUFUNC,DTINCLUSAO,DTULTATU,NRORGINCLUSAO,CDOPERINCLUSAO,NRORGULTATU,CDOPERULTATU,DSOBSALTSITFUNC,NRQTDIASITUFUNC,DTESTABSITUFUNC) 
				VALUES ((SELECT MAX(NRALTESITUFUNC) + 1 FROM GPE_ALTESITUFUNC WHERE NRORG = :NEW.NRORG)/*TO_NUMBER(V_PRXCODOCOR)*/, :NEW.NRORG, NULL, NULL, NULL, NULL, V_NRPROXSITUFUNC/*V_NRSITUFUNCM*/, :NEW.NRVINCULOM, :NEW.DTFIMPROGOCOR + 1, NULL, NULL, NULL, NULL, SYSDATE, SYSDATE, :NEW.NRORG, :NEW.CDOPERINCLUSAO, NULL, NULL, NULL, NULL, NULL);
		ELSIF UPDATING THEN
			/* Atualiza dados da situação funcional referente a ocorrência */
			UPDATE GPE_ALTESITUFUNC 
			SET DTINISITUFUNC = :NEW.DTINICIOPROGOCOR, DTFIMSITUFUNC = :NEW.DTFIMPROGOCOR,
				NRSITUFUNCM = V_NRSITUFUNCM, NRMOTIVOAFASTA = V_NRMOTIVOAFAST, DSOBSALTSITFUNC = :NEW.DSOBSERVACAO
			WHERE NRORG          = :OLD.NRORG
			AND NRVINCULOM     = :OLD.NRVINCULOM
			AND DTINISITUFUNC  = :OLD.DTINICIOPROGOCOR
			AND DTFIMSITUFUNC  = :OLD.DTFIMPROGOCOR;        
			/* Atualiza dados da situação funcional referente volta em atividade normal */
			UPDATE GPE_ALTESITUFUNC 
			SET DTINISITUFUNC = :NEW.DTFIMPROGOCOR + 1
			WHERE NRORG          = :OLD.NRORG
			AND NRVINCULOM     = :OLD.NRVINCULOM
			AND DTINISITUFUNC  = :OLD.DTFIMPROGOCOR + 1
			AND NRSITUFUNCM    = V_NRPROXSITUFUNC
			AND DTFIMSITUFUNC IS NULL;     
		END IF;                     
		END IF;
	ELSIF DELETING THEN    
			/* Exclui situação funcional referente a ocorrência */
			DELETE FROM GPE_ALTESITUFUNC
			WHERE NRORG          = :OLD.NRORG
			AND NRVINCULOM     = :OLD.NRVINCULOM
			AND DTINISITUFUNC  = :OLD.DTINICIOPROGOCOR
			AND DTFIMSITUFUNC  = :OLD.DTFIMPROGOCOR;
			/* Exclui situação funcional referente volta em atividade normal */
			DELETE FROM GPE_ALTESITUFUNC
			WHERE NRORG          = :OLD.NRORG
			AND NRVINCULOM     = :OLD.NRVINCULOM
			AND DTINISITUFUNC  = :OLD.DTFIMPROGOCOR + 1
			AND NRSITUFUNCM    = V_NRPROXSITUFUNC
			AND DTFIMSITUFUNC IS NULL;
	END IF;
  END IF;
END;
/